@@ -8,12 +8,42 @@ from django.conf import settings
 from django.contrib.admin.views.main import ChangeList, ALL_VAR, IS_POPUP_VAR, \
     ORDER_TYPE_VAR, ORDER_VAR, SEARCH_VAR
 from django.contrib.sites.models import Site
-from menus.utils import find_children
-
 
 
 COPY_VAR = "copy"
 
+
+def cache_tree_children(queryset):
+    """
+    For all items in the queryset, set the '_cached_children' attribute to a
+    list. This attribute is in turn used by the 'get_children' method on the
+    item, which would otherwise (if '_cached_children' is not set) cause a 
+    database query.
+    
+    The queryset MUST BE ORDERED BY 'lft', 'tree_id'! Otherwise this function
+    will raise a ValueError.
+    """
+    parents_dict = {}
+    lastleft = -1 # integrity check
+    lasttree = -1 # integrity check
+    for obj in queryset:
+        parents_dict[obj.pk] = obj
+        if obj.tree_id == lasttree and obj.lft < lastleft: # integrity check
+                raise ValueError('Objects passed in the wrong order, must be ordered by the mptt left attribute and tree id')
+        lastleft = obj.lft # integrity check
+        lasttree = obj.tree_id # integrity check
+        # set the '_cached_children' attribute
+        obj._cached_children = []
+        # get the parent of this object (if available) via parent_id
+        parent = parents_dict.get(obj.parent_id, None)
+        if parent:
+            # if there is a parent, append the current object to the _cached_children
+            # list of the parent. Since the objects are ordered by lft, tree_id
+            # the _cached_children attribute will always have been set by this
+            # function already.
+            parent._cached_children.append(obj)
+
+
 class CMSChangeList(ChangeList):
     '''
     Renders a Changelist - In our case it looks like a tree - it's the list of
@@ -50,7 +80,6 @@ class CMSChangeList(ChangeList):
                 self.root_query_set = self.root_query_set.filter(pk__in=permissions)
             self.real_queryset = True
             qs = qs.filter(site=self._current_site)
-        qs = qs.order_by('tree_id', 'parent', 'lft')
         return qs
     
     def is_filtered(self):
@@ -75,7 +104,8 @@ class CMSChangeList(ChangeList):
         site = self._current_site
         # Get all the pages, ordered by tree ID (it's convenient to build the 
         # tree using a stack now)
-        pages = self.get_query_set(request).drafts().order_by('tree_id', 'parent', 'lft').select_related()
+        pages = self.get_query_set(request).drafts().order_by('tree_id',  'lft').select_related()
+        
         
         # Get lists of page IDs for which the current user has 
         # "permission to..." on the current site. 
@@ -111,13 +141,21 @@ class CMSChangeList(ChangeList):
         ids = []
         root_pages = []
         pages = list(pages)
-        all_pages = pages[:]
+        all_pages = pages[:] # That is, basically, a copy.
         try:
             home_pk = Page.objects.drafts().get_home(self.current_site()).pk
         except NoHomeFound:
-            home_pk = 0    
+            home_pk = 0
+            
+        # Unfortunately we cannot use the MPTT builtin code for pre-caching
+        # the children here, because MPTT expects the tree to be 'complete'
+        # and otherwise complaints about 'invalid item order'
+        cache_tree_children(pages)
+        
         for page in pages:
-            children = []
+           
+
+            children = page.get_children()
 
             # note: We are using change_list permission here, because we must
             # display also pages which user must not edit, but he haves a 
@@ -133,7 +171,7 @@ class CMSChangeList(ChangeList):
                 # caching the permissions
                 page.permission_edit_cache = perm_edit_ids == Page.permissions.GRANT_ALL or page.pk in perm_edit_ids
                 page.permission_publish_cache = perm_publish_ids == Page.permissions.GRANT_ALL or page.pk in perm_publish_ids
-                page.permission_advanced_settings_cache = perm_publish_ids == Page.permissions.GRANT_ALL or page.pk in perm_advanced_settings_ids
+                page.permission_advanced_settings_cache = perm_advanced_settings_ids == Page.permissions.GRANT_ALL or page.pk in perm_advanced_settings_ids
                 page.permission_user_cache = request.user
             
             if settings.CMS_MODERATOR:
@@ -155,6 +193,8 @@ class CMSChangeList(ChangeList):
             if page.root_node or self.is_filtered():
                 page.last = True
                 if len(children):
+                    # TODO: WTF!?!
+                    # The last one is not the last... wait, what?
                     children[-1].last = False
                 page.menu_level = 0
                 root_pages.append(page)
@@ -163,10 +203,20 @@ class CMSChangeList(ChangeList):
                 else:
                     page.ancestors_ascending = []
                 page.home_pk_cache = home_pk
-                if not self.is_filtered():
-                    find_children(page, pages, 1000, 1000, [], -1, soft_roots=False, request=request, no_extended=True, to_levels=1000)
-                else:
-                    page.childrens = []
+            
+            # Because 'children' is the reverse-FK accessor for the 'parent'
+            # FK from Page->Page, we have to use wrong English here and set
+            # an attribute called 'childrens'. We are aware that this is WRONG
+            # but what should we do?
+            
+            # If the queryset is filtered, do NOT set the 'childrens' attribute
+            # since *ALL* pages will be in the 'root_pages' list and therefore
+            # be displayed. (If the queryset is filtered, the result is not a
+            # tree but rather a flat list).
+            if self.is_filtered():
+                page.childrens = []
+            else:
+                page.childrens = children
         
         # TODO: OPTIMIZE!!
         titles = Title.objects.filter(page__in=ids)
