@@ -1,48 +1,65 @@
 # -*- coding: utf-8 -*-
-from cms.exceptions import DontUsePageAttributeWarning
-from cms.models.placeholdermodel import Placeholder
-from cms.plugin_rendering import PluginContext, PluginRenderer
-from cms.utils.helpers import reversion_register
-from cms.utils.placeholder import get_page_from_placeholder_if_exists
+import os
+import warnings
 from datetime import datetime, date
+
 from django.conf import settings
 from django.core.exceptions import ValidationError, ObjectDoesNotExist
 from django.db import models
-from django.db.models.base import ModelBase, model_unpickle, \
-    simple_class_factory
+from django.db.models.base import (model_unpickle, simple_class_factory)
 from django.db.models.query_utils import DeferredAttribute
 from django.utils.translation import ugettext_lazy as _
-from os.path import join
-from publisher.mptt_support import Mptt, install_mptt
-import warnings
 
-class PluginModelBase(ModelBase):
+from cms.exceptions import DontUsePageAttributeWarning
+from cms.models.placeholdermodel import Placeholder
+from cms.plugin_rendering import PluginContext, render_plugin
+from cms.utils.helpers import reversion_register
+from cms.utils.placeholder import get_page_from_placeholder_if_exists
+
+from mptt.models import MPTTModel, MPTTModelBase
+
+
+class BoundRenderMeta(object):
+    def __init__(self, meta):
+        self.index = 0
+        self.total = 1
+        self.text_enabled = getattr(meta, 'text_enabled', False)
+
+
+class PluginModelBase(MPTTModelBase):
     """
-    Metaclass for all plugins.
+    Metaclass for all CMSPlugin subclasses. This class should not be used for
+    any other type of models.
     """
     def __new__(cls, name, bases, attrs):
-        render_meta = attrs.pop('RenderMeta', None)
-        if render_meta is not None:
-            attrs['_render_meta'] = render_meta()
-        attrs = install_mptt(cls, name, bases, attrs)
+        # remove RenderMeta from the plugin class
+        attr_meta = attrs.pop('RenderMeta', None)
+
+        # create a new class (using the super-metaclass)
         new_class = super(PluginModelBase, cls).__new__(cls, name, bases, attrs)
-        found = False
-        bbases = bases
-        while bbases:
-            bcls = bbases[0]
-            if bcls.__name__ == "CMSPlugin":
-                found = True
-                bbases = False
-            else:
-                bbases = bcls.__bases__  
-        if found:
-            if new_class._meta.db_table.startswith("%s_" % new_class._meta.app_label):
-                table = "cmsplugin_" + new_class._meta.db_table.split("%s_" % new_class._meta.app_label, 1)[1]
-                new_class._meta.db_table = table
-        return new_class 
+        
+        # if there is a RenderMeta in attrs, use this one
+        if attr_meta:
+            meta = attr_meta
+        else:
+            # else try to use the one from the superclass (if present)
+            meta = getattr(new_class, '_render_meta', None)
+        
+        # set a new BoundRenderMeta to prevent leaking of state
+        new_class._render_meta = BoundRenderMeta(meta)
+
+        # turn 'myapp_mymodel' into 'cmsplugin_mymodel' by removing the
+        # 'myapp_' bit from the db_table name.
+        if [base for base in bases if isinstance(base, PluginModelBase)]:
+            splitter = '%s_' % new_class._meta.app_label
+            splitted = new_class._meta.db_table.split(splitter, 1)
+            table_name = 'cmsplugin_%s' % splitted[1]
+            new_class._meta.db_table = table_name
+        
+        return new_class
          
     
-class CMSPlugin(Mptt):
+class CMSPlugin(MPTTModel):
     '''
     The base class for a CMS plugin model. When defining a new custom plugin, you should
     store plugin-instance specific information on a subclass of this class.
@@ -159,8 +176,7 @@ class CMSPlugin(Mptt):
                     raise ValidationError("plugin has no render_template: %s" % plugin.__class__)
             else:
                 template = None
-            renderer = PluginRenderer(context, instance, placeholder, template, processors)
-            return renderer.content
+            return render_plugin(context, instance, placeholder, template, processors)
         return ""
             
     def get_media_path(self, filename):
@@ -169,8 +185,9 @@ class CMSPlugin(Mptt):
             return pages[0].get_media_path(filename)
         else: # django 1.0.2 compatibility
             today = date.today()
-            return join(settings.CMS_PAGE_MEDIA_PATH, str(today.year), str(today.month), str(today.day), filename)
-            
+            return os.path.join(settings.CMS_PAGE_MEDIA_PATH,
+                str(today.year), str(today.month), str(today.day), filename)
+
     @property
     def page(self):
         warnings.warn(
@@ -222,6 +239,7 @@ class CMSPlugin(Mptt):
         new_plugin.tree_id = None
         new_plugin.lft = None
         new_plugin.rght = None
+        new_plugin.level = None
         if self.parent:
             pdif = self.level - plugin_tree[-1].level
             if pdif < 0:
@@ -246,6 +264,7 @@ class CMSPlugin(Mptt):
             plugin_instance.level = new_plugin.level
             plugin_instance.cmsplugin_ptr = new_plugin
             plugin_instance.language = target_language
+            plugin_instance.parent = new_plugin.parent
             plugin_instance.position = new_plugin.position # added to retain the position when creating a public copy of a plugin
             plugin_instance.save()
             old_instance = plugin_instance.__class__.objects.get(pk=self.pk)
