@@ -44,9 +44,6 @@ class VideoPlugin(CMSPluginBase):
                 'classes': ('collapse',),
             }),
         ]
-    
-    class PluginMedia:
-        js = ('http://ajax.googleapis.com/ajax/libs/swfobject/2.1/swfobject.js',)
         
     def render(self, context, instance, placeholder):
         context.update({
