@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
+from cms.api import create_page, create_title, publish_page, add_plugin
 from cms.middleware.multilingual import patch_response
-from cms.test.testcases import CMSTestCase
+from cms.test_utils.testcases import CMSTestCase
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
 import urllib
 
 
 class MultilingualTestCase(CMSTestCase):
-    def test_01_multilingual_url_middleware(self):
+    def test_multilingual_url_middleware(self):
         """
         Test the Multilingual URL Middleware correctly handles the various ways
         one can write URLs in HTML.
@@ -41,29 +42,24 @@ class MultilingualTestCase(CMSTestCase):
         expected = '<form action="/en/url/">'
         self.assertEqual(output, expected)
         
-    def test_02_multilingual_page(self):
-        page = self.create_page()
-        self.create_title(
-            page=page,
-            title=page.get_title(),
-            slug=page.get_slug(),
-            language='de'
-        )
+    def test_multilingual_page(self):
+        page = create_page("mlpage", "nav_playground.html", "en")
+        create_title("de", page.get_title(), page, slug=page.get_slug())
         page.rescan_placeholders()
         page = self.reload(page)
         placeholder = page.placeholders.all()[0]
-        self.add_plugin(placeholder=placeholder, language='de')
-        self.add_plugin(placeholder=placeholder, language='en')
+        add_plugin(placeholder, "TextPlugin", 'de', body="test")
+        add_plugin(placeholder, "TextPlugin", 'en', body="test")
         self.assertEqual(placeholder.cmsplugin_set.filter(language='de').count(), 1)
         self.assertEqual(placeholder.cmsplugin_set.filter(language='en').count(), 1)
         user = User.objects.create_superuser('super', 'super@django-cms.org', 'super')
-        page = self.publish_page(page, approve=True, user=user)
+        page = publish_page(page, user, True)
         public = page.publisher_public
         placeholder = public.placeholders.all()[0]
         self.assertEqual(placeholder.cmsplugin_set.filter(language='de').count(), 1)
         self.assertEqual(placeholder.cmsplugin_set.filter(language='en').count(), 1)
 
-    def test_03_multiple_reverse_monkeypatch(self):
+    def test_multiple_reverse_monkeypatch(self):
         """
         This test is not very well behaved, every following
         test that uses reverse will fail with a RuntimeException.
