@@ -1,12 +1,16 @@
 # -*- coding: utf-8 -*-
-from django.template.loader import render_to_string
 from django.core.mail import EmailMultiAlternatives
-from django.contrib.sites.models import Site
-from cms.utils.urlutils import urljoin
+from django.template.loader import render_to_string
+from django.utils.translation import ugettext_lazy as _
+
 from django.contrib import admin
+from django.contrib.sites.models import Site
+
+from cms.utils.urlutils import urljoin
 
 def send_mail(subject, txt_template, to, context=None, html_template=None, fail_silently=True):
-    """Multipart message helper with template rendering.
+    """
+    Multipart message helper with template rendering.
     """
     site = Site.objects.get_current()
     
@@ -23,4 +27,19 @@ def send_mail(subject, txt_template, to, context=None, html_template=None, fail_
     if html_template:
         body = render_to_string(html_template, context)
         message.attach_alternative(body, 'text/html')
-    message.send(fail_silently=fail_silently)
\ No newline at end of file
+    message.send(fail_silently=fail_silently)
+
+def mail_page_user_change(user, created=False, password=""):
+    """
+    Send email notification to given user.
+    Used it PageUser profile creation/update.
+    """
+    if created:
+        subject = _('CMS - your user account was created.')
+    else:
+        subject = _('CMS - your user account was changed.')
+    send_mail(subject, 'admin/cms/mail/page_user_change.txt', [user.email], {
+        'user': user,
+        'password': password or "*" * 8,
+        'created': created,
+    }, 'admin/cms/mail/page_user_change.html')
